@@ -1,11 +1,11 @@
 #
-# $Id: Simple.pm,v 1.10 2006/12/09 18:37:36 gomor Exp $
+# $Id: Simple.pm,v 1.5 2006/12/03 16:24:46 gomor Exp $
 #
 package Net::Frame::Simple;
 use warnings;
 use strict;
 
-our $VERSION = '1.00';
+our $VERSION = '1.00_01';
 
 require Class::Gomor::Array;
 our @ISA = qw(Class::Gomor::Array);
@@ -42,7 +42,7 @@ sub _gettimeofday {
 sub new {
    my $self = shift->SUPER::new(
       timestamp  => _gettimeofday(),
-      firstLayer => NF_LAYER_UNKNOWN,
+      firstLayer => NP_LAYER_UNKNOWN,
       layers     => [],
       @_,
    );
@@ -51,16 +51,6 @@ sub new {
    $self;
 }
 
-sub newFromDump {
-   my $self = shift;
-   my ($h) = @_;
-   $self->new(
-      timestamp  => $h->{timestamp},
-      firstLayer => $h->{firstLayer},
-      raw        => $h->{raw},
-   );
-}
-
 # If there are multiple layers of the same type, the upper will be kept
 sub _setRef {
    my $self = shift;
@@ -73,7 +63,7 @@ sub unpack {
 
    my $encapsulate = $self->[$__firstLayer];
 
-   if ($encapsulate eq NF_LAYER_UNKNOWN) {
+   if ($encapsulate eq NP_LAYER_UNKNOWN) {
       print("Unable to unpack frame from this layer type.\n");
       return undef;
    }
@@ -83,30 +73,20 @@ sub unpack {
    my $raw       = $self->[$__raw];
    my $rawLength = length($raw);
    my $oRaw      = $raw;
-   my $prevLayer;
    # No more than a thousand nested layers, maybe should be a parameter
    for (1..1000) {
       last unless $raw;
 
-      my $layer = 'Net::Frame::'.$encapsulate;
+      my $layer  = 'Net::Frame::'.$encapsulate;
       (my $module = $layer) =~ s/::/\//g;
       eval { require "$module.pm" };
       if ($@) {
          print("*** Net::Frame::$encapsulate module not found.\n".
                "*** Either install it (if avail), or implement it.\n".
                "*** You can also send the pcap file to perl\@gomor.org.\n");
-               # XXX: use debug feature from Class::Gomor
-               #"*** You can also send the pcap file to perl\@gomor.org.\n".
-               #"*** ERROR: $@\n");
-
-         if ($prevLayer) {
-            $prevLayer->nextLayer(NF_LAYER_NOT_AVAILABLE);
-         }
-
          last;
       }
-      my $l = $layer->new(raw => $raw)->unpack
-         or last;
+      my $l = $layer->new(raw => $raw)->unpack;
 
       $encapsulate = $l->encapsulate;
       $raw         = $l->payload;
@@ -117,14 +97,13 @@ sub unpack {
 
       last unless $encapsulate;
 
-      if ($encapsulate eq NF_LAYER_UNKNOWN) {
+      if ($encapsulate eq NP_LAYER_UNKNOWN) {
          print("Unable to unpack next layer, not yet implemented in layer: ".
                "$n:@{[$l->layer]}\n");
          last;
       }
 
-      $prevLayer = $l;
-      $oRaw      = $raw;
+      $oRaw = $raw;
    }
 
    if (@layers > 0) {
@@ -142,6 +121,13 @@ sub unpack {
 sub computeLengths {
    my $self = shift;
 
+   my $params;
+   for my $l (@{$self->[$__layers]}) {
+      do { $params->{ICMPv4_Type} = $l; next } if $l->layer =~ /^ICMPv4./;
+   }
+
+   my $icmp4Type = $params->{ICMPv4_Type};
+
    if (exists $self->[$__ref]->{IPv4}) {
       my $ip4 = $self->[$__ref]->{IPv4};
       if (exists $self->[$__ref]->{TCP}) {
@@ -158,10 +144,10 @@ sub computeLengths {
             payloadLength => $udp->getLength + $udp->getPayloadLength,
          });
       }
-      elsif (exists $self->[$__ref]->{ICMPv4}) {
+      elsif (exists $self->[$__ref]->{ICMPv4} && $icmp4Type) {
          my $icmp4 = $self->[$__ref]->{ICMPv4};
          $ip4->computeLengths({
-            payloadLength => $icmp4->getLength,
+            payloadLength => $icmp4->getLength + $icmp4Type->getLength,
          });
       }
    }
@@ -172,6 +158,13 @@ sub computeLengths {
 sub computeChecksums {
    my $self = shift;
 
+   my $params;
+   for my $l (@{$self->[$__layers]}) {
+      do { $params->{ICMPv4_Type} = $l; next } if $l->layer =~ /^ICMPv4./;
+   }
+
+   my $icmp4Type = $params->{ICMPv4_Type};
+
    if (exists $self->[$__ref]->{IPv4}) {
       my $ip4 = $self->[$__ref]->{IPv4};
       if (exists $self->[$__ref]->{ETH}) {
@@ -192,8 +185,10 @@ sub computeChecksums {
             dst  => $ip4->dst,
          });
       }
-      elsif (exists $self->[$__ref]->{ICMPv4}) {
-         $self->[$__ref]->{ICMPv4}->computeChecksums;
+      elsif (exists $self->[$__ref]->{ICMPv4} && $icmp4Type) {
+         $self->[$__ref]->{ICMPv4}->computeChecksums({
+            payload => $icmp4Type,
+         });
       }
    }
 
@@ -231,12 +226,6 @@ sub _getPadding {
    # Last layer has no payload, so no padding
    return unless $last->payload;
 
-   # If last layer has unknown layer payload, we do not 
-   # try to get padding
-   if ($last->nextLayer eq NF_LAYER_NOT_AVAILABLE) {
-      return;
-   }
-
    my $tLen = 0;
    for my $l (@{$self->[$__layers]}) {
       if ($l->layer eq 'IPv4') {
@@ -255,18 +244,8 @@ sub _getPadding {
 
    my $pLen    = ($rawLength > $tLen) ? ($rawLength - $tLen) : 0;
    my $padding = substr($self->[$__raw], $tLen, $pLen);
-
    $self->[$__padding] = $padding;
-
-   # Now, split padding between true padding and true payload
-   my $payloadLength = length($last->payload);
-   if ($payloadLength > $pLen) {
-      my $payload = substr($last->payload, 0, ($payloadLength - $pLen));
-      $last->payload($payload);
-   }
-   else {
-      $last->payload(undef);
-   }
+   $last->payload(undef);
 }
 
 sub send {
@@ -275,7 +254,7 @@ sub send {
    $oWrite->send($self->[$__raw]);
 }
 
-sub reSend { my $self = shift; $self->send(shift()) unless $self->[$__reply] }
+sub reSend { my $self = shift; $self->send unless $self->[$__reply] }
 
 sub _searchCanMatchLayer {
    my $self = shift;
@@ -317,16 +296,7 @@ sub _recv {
    for my $this ($oDump->getFramesFor($self)) {
       next unless $this->[$__timestamp] ge $self->[$__timestamp];
 
-      # We must put ICMPv4 before, because the other will 
-      # always match for UDP.
-      if (exists $this->[$__ref]->{ICMPv4}
-      &&  (exists $this->[$__ref]->{UDP} || exists $this->[$__ref]->{TCP})) {
-         if (exists $this->[$__ref]->{$layer->layer}) {
-            return $this
-               if $this->[$__ref]->{$layer->layer}->getKey eq $layer->getKey;
-         }
-      }
-      elsif (exists $this->[$__ref]->{$layer->layer}) {
+      if (exists $this->[$__ref]->{$layer->layer}) {
          return $this if $layer->match($this->[$__ref]->{$layer->layer});
       }
    }
@@ -344,7 +314,11 @@ sub recv {
    # Is there anything waiting ?
    my $h = $oDump->next or return undef;
 
-   my $oSimple = Net::Frame::Simple->newFromDump($h);
+   my $oSimple = Net::Frame::Simple->new(
+      raw        => $h->{raw},
+      firstLayer => $h->{firstLayer},
+      timestamp  => $h->{timestamp},
+   );
    $oDump->store($oSimple);
 
    if (my $reply = $self->_recv($oDump)) {
@@ -386,11 +360,16 @@ sub print {
 sub dump {
    my $self = shift;
 
-   my $raw = '';
-   $raw .= $_->raw for @{$self->[$__layers]};
-   $raw .= $self->[$__padding] if $self->[$__padding];
+   my $str = '';
+   for (@{$self->[$__layers]}) {
+      $str .= $_->dump."\n";
+   }
 
-   CORE::unpack('H*', $raw);
+   if ($self->[$__padding]) {
+      $str .= 'Padding: '.CORE::unpack('H*', $self->[$__padding])."\n";
+   }
+
+   $str;
 }
 
 1;
@@ -403,93 +382,89 @@ Net::Frame::Simple - frame crafting made easy
 
 =head1 SYNOPSIS
 
-   # We build a TCP SYN
-   my $src    = '192.168.0.10';
-   my $target = '192.168.0.1';
-   my $port   = 22;
+   require Net::Packet::Frame;
 
-   use Net::Frame::Simple;
-   use Net::Frame::IPv4;
-   use Net::Frame::TCP;
-
-   my $ip4 = Net::Frame::IPv4->new(
-      src => $src,
-      dst => $target,
-   );
-   my $tcp = Net::Frame::TCP->new(
-      dst     => $port,
-      options => "\x02\x04\x54\x0b",
-      payload => 'test',
+   # Because we passed a layer 3 object, a Net::Packet::DescL3 object 
+   # will be created automatically, by default. See Net::Packet::Env 
+   # regarding changing this behaviour. Same for Net::Packet::Dump.
+   my $frame = Net::Packet::Frame->new(
+      l3 => $ipv4,  #Net::Packet::IPv4 object
+      l4 => $tcp,   #Net::Packet::TCP object
+                    # (here, a SYN request, for example)
    );
 
-   my $oSimple = Net::Frame::Simple->new(
-      layers => [ $ip4, $tcp ],
-   );
+   # Without retries
+   $frame->send;
+   sleep(3);
+   if (my $reply = $frame->recv) {
+      print $reply->l3->print."\n";
+      print $reply->l4->print."\n";
+   }
 
-   # Now, the frame is ready to be send to the network
-   # We open a sender object, and a retriever object
-   use Net::Write::Layer3;
-   use Net::Frame::Dump::Online;
+   # Or with retries
+   for (1..3) {
+      $frame->reSend;
 
-   my $oWrite = Net::Write::Layer3->new(dst => $target);
-   my $oDump  = Net::Frame::Dump::Online->new(dev => $oDevice->dev);
-   $oDump->start;
-   $oWrite->open;
-
-   # We send the frame
-   $oSimple->send($oWrite);
-
-   # And finally, waiting for the response
-   until ($oDump->timeout) {
-      if (my $recv = $oSimple->recv($oDump)) {
-         print "RECV:\n".$recv->print."\n";
-         last;
+      until ($Env->dump->timeout) {
+         if (my $reply = $frame->recv) {
+            print $reply->l3->print."\n";
+            print $reply->l4->print."\n";
+            last;
+         }
       }
    }
 
-   $oWrite->close;
-   $oDump->stop;
-
 =head1 DESCRIPTION
 
-This module is part of B<Net::Frame> frame crafting framework. It is totally optional, but can make playing with the network far easier.
+In B<Net::Packet>, each sent and/or received frame is parsed and converted into a B<Net::Packet::Frame> object. Basically, it encapsulates various layers (2, 3, 4 and 7) into an object, making it easy to get or set information about it.
 
-Basically, it hides the complexity of frame forging, sending, and receiving, by providing helper methods, which will analyze internally how to assemble frames and find responses to probes.
+When you create a frame object, a B<Net::Packet::Desc> object is created if none is found in the default B<$Env> object (from B<Net::Packet> module), and a B<Net::Packet::Dump> object is also created if none is found in this same B<$Env> object. You can change this beheaviour, see B<Net::Packet::Env>.
 
-For example, it will take care of computing lengths and checksums, and matching a response frame to the requesting frame.
+Two B<new> invocation method exist, one with attributes passing, another with B<raw> attribute. This second method is usually used internally, in order to unpack received frame into all corresponding layers.
 
 =head1 ATTRIBUTES
 
 =over 4
 
+=item B<env>
+
+Stores the B<Net::Packet::Env> object. The default is to use B<$Env> from B<Net::Packet>. So, you can send/recv frames to/from different environements.
+
 =item B<raw>
 
-Where the packed frame will be stored, or used to unpack a raw string taken from the network (or elsewhere).
-
-=item B<timestamp>
-
-The frame timestamp.
-
-=item B<firstLayer>
-
-We cannot know by which layer a frame begins, so this tells how to start unpacking a raw data.
+Pass this attribute when you want to decode a raw string captured from network. Usually used internally.
 
 =item B<padding>
 
-Sometimes, frames are padded to achieve 60 bytes in length. The padding will be stored here, or if you craft a frame, you can manually add your own padding.
+In Ethernet world, a frame should be at least 60 bytes in length. So when you send frames at layer 2, a padding is added in order to achieve this length, avoiding a local memory leak to network. Also, when you receive a frame from network, this attribute is filled with what have been used to pad it. This padding feature currently works for IPv4 and ARP frames.
+
+=item B<l2>
+
+Stores a layer 2 object. See B<Net::Packet> for layer 2 classes hierarchy.
+
+=item B<l3>
+
+Stores a layer 3 object. See B<Net::Packet> for layer 3 classes hierarchy.
+
+=item B<l4>
+
+Stores a layer 4 object. See B<Net::Packet> for layer 4 classes hierarchy.
+
+=item B<l7>
+
+Stores a layer 7 object. See B<Net::Packet::Layer7>.
 
 =item B<reply>
 
-When the B<recv> method is called, and a corresponding reply has been found, it is stored here.
+When B<recv> method has been called on a frame object, and a corresponding reply has been catched, a pointer is stored in this attribute.
 
-=item B<layers>
+=item B<timestamp>
 
-This one is an arrayref. It will store all layers to use within the B<Net::Frame::Simple> object.
+When a frame is packed/unpacked, the happening time is stored here.
 
-=item B<ref>
-
-This is a hashref that stores all layers. The key is the layer type (example: TCP: B<$oSimple->ref->{TCP}>). If the frame contains multiple layers of the same type, only the one found at upper level will be kept (in fact, the latest analyzed one, aka LIFO).
+=item B<encapsulate>
 
+Give the type of the first encapsulated layer. It is a requirement to parse a user provided raw string.
 
 =back
 
@@ -497,62 +472,108 @@ This is a hashref that stores all layers. The key is the layer type (example: TC
 
 =over 4
 
-=item B<new> (hash)
+=item B<new>
 
-Object constructor. You can pass attributes in a hash as a parameter.
+Object constructor. If a B<$Env->desc> object does not exists, one is created by analyzing attributes (so, either one of B<Net::Packet::DescL2>, B<Net::Packet::DescL3>. B<Net::Packet::DescL4> cannot be created automatically for now). The same behaviour is true for B<$Env->dump> object. You can change this default creation behaviour, see B<Net::Packet::Env>. Default values:
 
-=item B<newFromDump> (hashref)
+timestamp: gettimeofday(),
 
-When B<Net::Frame::Dump> B<next> method is called, and there is a frame waiting, it returns a hashref with specific values. You can directly use it as a parameter for this method, which will create a new B<Net::Frame::Simple> object.
+env:       $Env
 
-=item B<computeLengths>
+=item B<getLengthFromL7>
 
-This one hides the manual hassle of calling B<computeLengths> method for each layers. It takes no parameter, it will know internally what to do.
+=item B<getLengthFromL4>
 
-=item B<computeChecksums>
+=item B<getLengthFromL3>
 
-Same as above, but for checksums. you MUST call the previous one before this one.
+=item B<getLengthFromL2>
 
-=item B<pack>
+Returns the raw length in bytes from specified layer.
 
-Will pack all layers to to B<raw> attribute, ready to be sent to the network.
+=item B<getLength>
+
+Alias for B<getLengthFromL3>.
 
 =item B<unpack>
 
-Will unpack a raw string from the B<raw> attribute into respective layers.
+Unpacks the raw string from network into various layers. Returns 1 on success, undef on failure.
 
-=item B<getKey>
+=item B<pack>
 
-=item B<getKeyReverse>
+Packs various layers into the raw string to send to network. Returns 1 on success, undef on failure.
 
-These two methods are basically used to increase the speed when using B<recv> method.
+=item B<send>
 
-=item B<recv> (Net::Frame::Dump object)
+On the first send invocation in your program, the previously created B<Net::Packet::Dump> object is started (if available). That is, packet capturing is run. The B<timestamp> attribute is set to the sending time. The B<env> attribute is used to know where to send this frame.
 
-When you want to search for the response of your probe, you call it by specifying from which B<Net::Frame::Dump> object to search. It then returns a B<Net::Frame::Simple> object if a match is found, or undef if not.
+=item B<reSend>
 
-=item B<send> (Net::Write object)
+Will call B<send> method if no frame has been B<recv>'d, that is the B<reply> attribute is undef.
 
-Will send to the B<Net::Write> object the raw string describing the B<Net::Frame::Simple> object.
+=item B<getFilter>
 
-=item B<reSend> (Net::Write object)
+Will return a string which is a pcap filter, and corresponding to what you should receive compared with the frame request.
 
-You can also B<reSend> the frame, it will only rewrite it to the network if no B<reply> has already been found.
+=item B<recv>
+
+Searches B<framesSorted> or B<frames> from B<Net::Packet::Dump> for a matching response. If a reply has already been received (that is B<reply> attribute is already set), undef is returned. It no reply is received, return undef, else the B<Net::Packet::Frame> response.
 
 =item B<print>
 
-Prints all layers in human readable format.
+Just returns a string in a human readable format describing attributes found in the layer.
 
 =item B<dump>
 
-Dumps the B<raw> string in hexadecimal format.
+Just returns a string in hexadecimal format which is how the layer appears on the network.
+
+=item B<isEth>
+
+=item B<isRaw>
+
+=item B<isNull>
+
+=item B<isSll>
+
+=item B<isPpp>
+
+=item B<isArp>
+
+=item B<isIpv4>
+
+=item B<isIpv6>
+
+=item B<isIp> - either IPv4 or IPv6
+
+=item B<isPpplcp>
+
+=item B<isVlan>
+
+=item B<isPppoe>
+
+=item B<isLlc>
+
+=item B<isTcp>
+
+=item B<isUdp>
+
+=item B<isIcmpv4>
+
+=item B<isIcmp> - currently only ICMPv4
+
+=item B<isCdp>
+
+=item B<isStp>
+
+=item B<isOspf>
+
+=item B<isIgmpv4>
+
+=item B<is7>
+
+Returns 1 if the B<Net::Packet::Frame> is of specified layer, 0 otherwise.
 
 =back
 
-=head1 SEE ALSO
-
-L<Net::Write>, L<Net::Frame::Dump>
-
 =head1 AUTHOR
 
 Patrice E<lt>GomoRE<gt> Auffret
